#ifndef CIMAGE_H	
	#define CIMAGE_H
	#include "windows.h"
	#include "math.h"
	#include "resource.h"
	#include <stdio.h>

	#define PI	3.141592654
	#define NB_BUFFER		25
	#define FREEMEM(x) VirtualFree(x, 0, MEM_RELEASE)
	#define ALLOCMEM(size) VirtualAlloc(0, size, MEM_COMMIT|MEM_RESERVE, PAGE_READWRITE)
	

	#define RED		7
	#define GREEN	6
	#define BLUE	5
	#define BIN		3
	#define GRAY	2 
	#define RGBi	1

	typedef struct TAG_DIMENSION
	{
		int cx,cy;
	}Dimension;


	class CImage
	{
	public:
		//Constructeurs et destructeur
		CImage();
		CImage(char *FileName);
		CImage(CImage *Other);
		~CImage(){Nettoyage();}

		//Chargement d'une image
		bool LoadBmp(char *FileName);
		bool LoadRaw(char *FileName);
		bool ReadRaw(char *FileName,int _Width,int _Height,int NbBit_Par_Pixel);
		bool SelectPictureLoad(HWND hwnd);
		
		int **Convert_2D(int Type);
		bool Convert_2D_to_1D(int **Tab2D);

		//Sauvegarde d'une image
		void Save(HWND hwnd, char *szFileName);
		bool SelectPictureSave(HWND hwnd=0);
		
		//Suppression de l'image
		void Delete(){Nettoyage();}
		

		//Copie de l'image
		void Copy(const CImage *Cpy);
		void CopyProp(const CImage *Cpy);
		void Alloc(int Width,int Height,int Planes,int BitsPixel);
		
		//Dessin de l'image
		void DrawImage(HDC hdc,int cx,int cy,double Zoom);
		
		//Opration arithmtique sur les images
		CImage &CImage::operator-=(const CImage &Other);
		CImage &CImage::operator+=(const CImage &Other);
		void Soustraction	(CImage *Other,CImage *ImgDest=0);
		void Addition2		(CImage *Other,CImage *ImgDest=0);
		void Multiplication	(CImage *Other,CImage *ImgDest=0);
		bool Inversion(CImage *ImgDest=0);

		//Opration sur les images binaires
		void AND (CImage *Other,CImage *ImgDest=0);
		void NAND(CImage *Other,CImage *ImgDest=0);
		void OR  (CImage *Other,CImage *ImgDest=0);
		void NOR (CImage *Other,CImage *ImgDest=0);
		bool NOT (CImage *ImgDest=0);

		//Fonctions asseceurs
		Dimension GetDimension();
		int	GetType(){return ImgType;}
		double GetPixel(int i,int j,int Type);
		void SetPixel(int i,int j,int R,int G,int B);

		bool Fill(int R,int G,int B);
		bool DrawHorizontalLine(int Y,int R,int G,int B);
		bool DrawVerticalLine(int X,int R,int G,int B);
		int  NbPixel(int R,int G,int B);

		//Modification de l'intensit de l'image
		void ModifEclairement(int Luminance);
		bool EgalisationContraste(int Type,CImage *ImgDest=0);
		bool ContrasteAuto(int Type,CImage *ImgDest=0);
		bool ConvertRGB2Gray(CImage *ImgDest=0);

		//Mesure sur l'image
		int* Histogramme(int Type);
		double *HistogrammeNorm(int Type);
		void ChercherMinMax(int Type,int *Min,int *Max);
		
		//FILTRES 
		//Detection de contours
		bool Sobel		(int Type,CImage *ImgDest=0);
		bool Prewitt	(int Type,CImage *ImgDest=0);
		bool LaplaceI	(int Type,CImage *ImgDest=0);
		bool LaplaceII	(int Type,CImage *ImgDest=0);
		//Passe bas
		bool Moyenne	(int Type,CImage *ImgDest=0);
		bool Median		(int Type,int n,int NumPixel=4,CImage *ImgDest=0);
		//Passe Haut
		bool PasseHautv4(int Type,CImage *ImgDest=0);
		bool PasseHautv8(int Type,CImage *ImgDest=0);
		//Autre
		bool Convolution(int Type,CImage *ImgDest=0,int KernelLenght=25,int Normalisation=-99,int *Noyau=0);
		bool Repoussage(int Type,CImage *ImgDest=0);
		void DiscretFourier(int Dir);
		
		
		//Morphologie mathmatique
		bool Erosion     (bool *Noyau,int KernelLenght,bool BordureColor,CImage *ImgDest=0);
		bool Dilatation  (bool *Noyau,int KernelLenght,bool BordureColor,CImage *ImgDest=0);
		bool Fermeture   (bool *Noyau,int KernelLenght,bool BordureColor,CImage *ImgDest=0);
		bool Ouverture   (bool *Noyau,int KernelLenght,bool BordureColor,CImage *ImgDest=0);
		bool BlackTopHat (bool *Noyau,int KernelLenght,bool BordureColor,CImage *ImgDest=0);
		bool WhiteTopHat (bool *Noyau,int KernelLenght,bool BordureColor,CImage *ImgDest=0);
		bool ErosionNG   (bool *Noyau,int KernelLenght,bool BordureColor,CImage *ImgDest=0);
		bool DilatationNG(bool *Noyau,int KernelLenght,bool BordureColor,CImage *ImgDest=0);
		bool GradientMor (bool *Noyau,int KernelLenght,bool BordureColor,CImage *ImgDest=0);


		bool DistanceMorphologique(int Taille,bool Bordure,CImage *ImgDest=0);
		bool morReconstruction(CImage *Marqueur,int Taille,CImage *ImgDest=0);
		bool morCloseHole(int Taille,CImage *ImgDest=0);

		//Seuillage bianire
		bool Threashold		(int Seuil,CImage *ImgDest=0);
		int thrMaxVariance	(BOOL Mahalanobis);
		int thrMaxEntropie	();
		int thrLargeurMiHauteur(int Coeff);
		int thrOtsu			();
		int thrPourcentage	(double Pourcent);
		int thrJoh			();
		int thrMinError		();
		int thrFuzzy		();
		int thrFisher		();
		int thrTwoPeak		();

		bool Minimum(int Type,CImage *Other,CImage *ImgDest=0);
		bool Maximum(int Type,CImage *Other,CImage *ImgDest=0);
	
		bool LPE(int Label,int ImgDest,int Neighborhood);
		bool lpeFrontiere(CImage  *ImgContour,CImage  *ImgDest=0);
		bool FindMinimum(CImage *ImgDest);
		bool Superposition(CImage *ImgFond);
              /*  int min(int x, int y);
                 int max(int x, int y);   */

	private:
		
		HBITMAP hBmp;
		int Height,Width;
		UCHAR *ucBits;
		int ImgType;
		WORD Planes,BitsPixel;

		void Nettoyage();

		void CImage::LPE_Init(int Label,int Neighborhood);
		void CImage::LabelProcessing(int i,int j,int Num);
		void CImage::Labeliser();
		void Init_Queue();
		bool CImage::IsEmptyFAH();
		bool CImage::IsEmptyFAH(int num);
		void CImage::NeighborProcess(POINT PtNeighbor,POINT Pt,int n);
		void CImage::FindNeigbor(POINT Pt,int n);
		int CImage::SelectNextQueue();
		void CImage::PixelProcess(POINT Pt,int n);
		void PixelCollision();
	};
	UCHAR Limit(int Val);
	UCHAR LimitABS(int Val);
	void CreateBMPFile(HWND hwnd, LPTSTR pszFile, PBITMAPINFO pbi,HBITMAP hBMP, HDC hDC) ;
	PBITMAPINFO CreateBitmapInfoStruct(HWND hwnd, HBITMAP hBmp);
	
	extern CImage Img[NB_BUFFER];
	extern CImage ImgOldSrc,ImgOldDest;
	extern int NumBufferDest,NumBufferSrc,DoUnDo;
	
	DWORD WINAPI cells(LPVOID lpParam );
	DWORD WINAPI demo(LPVOID lpParam);
#endif

